# OB-USP-AGENT Test Specification

Most of the functionality in obuspa can be tested via its command line tool i.e., with 
-c option. Each method can be broken down into an individual test case to show full coverage is
achieved.

# Sections
* [Preqreuisites](#prerequisites)
* [Test Suites](#test-suites)
* [Functional API Tests](#functional-api-tests)
* [Unit Tests](#unit-tests)
* [Functional Tests](#functional-tests)


## Prerequisites

The prerequisite for the uspd test suites is that libbbfdm and ubusd has to be
built for the TEST platform, a version prepared to publish dummy data for get
API, and record set API to a test logfile at `/tmp/test.log`.

| Dependency |                   Link                   | License  |
| :---		 | :---                                     | :---	   |
| ---------- | ---------------------------------------- | -------- |
| ubusd      | https://git.openwrt.org/project/ubus.git | LGPL 2.1 |
| uspd       | https://git.openwrt.org/project/ubus.git | LGPL 2.1 |
| libbbfdm   | https://dev.iopsys.eu/iopsys/bbf.git     | LGPL 2.1 |


## Test Suites

The obuspa build pipe has three test suites, a functional-api suite, unit test suite
and functional test suite.

### Functional API Tests

The functional API tests consists of individual test suites, one per object
Ubus under test. The functional API tests uses validation with the help of 
test-controller and command-line interface tool to invoke a method.

#### obuspa

| Execution ID	| Method      	 | Description 			                  | Function ID Coverage					|
| :---			| :--- 	  		 | :---					                  | :---									|
| 1				| help           | No argument	  		                  | [1](./functionspec.md#help)     		|
| 2				| version        | No argument	  		                  | [2](./functionspec.md#version)     		|
| 3				| get  	         | With path argument                     | [3](./functionspec.md#get)      		|
| 4				| set	         | With path and value arguments          | [4](./functionspec.md#set)	            |
| 5				| add  	         | With path argument	                  | [5](./functionspec.md#add)		        |
| 6	    		| del          	 | With path argument		              | [6](./functionspec.md#del)	            |
| 7				| operate     	 | With path, action and input arguments  | [7](./functionspec.md#operate)			|
| 8				| instances   	 | With path argument  	                  | [8](./functionspec.md#instances)	    |
| 9				| show           | With Specified string	              | [9](./functionspec.md#show)	            |
| 10			| dump           | With Specified string	              | [10](./functionspec.md#dump)	        |
| 11			| dbget  	     | With data basepaths array argument	  | [11](./functionspec.md#dbget)		    |
| 12			| dbset  	     | With data basepaths array argument	  | [12](./functionspec.md#dbset)		    |
| 13			| dbdel  	     | With data basepaths array argument	  | [13](./functionspec.md#dbdel)		    |
| 14			| verbose    	 | With paths array argument 	          | [14](./functionspec.md#verbose)	        |
| 15			| prototrace   	 | With paths array argument 	          | [15](./functionspec.md#prototrace)	    |

### Unit Tests

The obuspa unit tests performed through validate_cmd.sh via validate_output,
directly from the source code, which is compiled into a shared library.
This means testing the obuspa locally eith -c option and verifying the output 
 with pre-defined validated output file. Validation files can be placed in
validate_output as validate_xxxx (where xxxx represents the unit test or bug number). 

| Execution ID	| Method	| Test Case Name							| Function ID Coverage						|
| :---			| :--- 		| :---										| :---										|
| 1				| get  	    | [validate_2802](#validate_2802)           | [1](./functionspec.md#get)      		    |
| 2				| get  	    | [validate_2925](#validate_2925)           | [2](./functionspec.md#get)      		    |
| 3				| add  	    | [validate_2912_1](#validate_2912_1)       | [3](./functionspec.md#add)      		    |
| 4				| set  	    | [validate_2912_3](#validate_2912_3)       | [4](./functionspec.md#set)      		    |


#### validate_2802

##### Description

Tests the obuspa method `get` for test case 2802, 
for path "Device.DeviceInfo.ProcessStatus.Process.[PID==1]."
[get](./functionspec.md#get).

##### Test Steps

Issuing a get to a client from the obuspa test platform, with cli option.

Read the logfile and verify that the output argument was
accurately logged.

##### Test Expected Results

The expected result is for the log file to have recorded a call to the
get handler function in obuspa.

````bash
Device.DeviceInfo.ProcessStatus.Process.1.CPUTime => 3000
Device.DeviceInfo.ProcessStatus.Process.1.Command => /sbin/procd
Device.DeviceInfo.ProcessStatus.Process.1.PID => 1
Device.DeviceInfo.ProcessStatus.Process.1.Priority => 20
Device.DeviceInfo.ProcessStatus.Process.1.Size => 1444
Device.DeviceInfo.ProcessStatus.Process.1.State => Sleeping
````

#### validate_2925

##### Description

Tests the obuspa method `get` for test case 2925, 
for path "Device.DeviceInfo.ProcessStatus.Process.[Size>1500]."
[get](./functionspec.md#get).

##### Test Steps

Issuing a get to a client from the obuspa test platform, with cli option.

Read the logfile and verify that the output argument was
accurately logged.

##### Test Expected Results

The expected result is for the log file to have recorded a call to the
get handler function in obuspa.

````bash
Device.DeviceInfo.ProcessStatus.Process.50.CPUTime => 0
Device.DeviceInfo.ProcessStatus.Process.50.Command => /sbin/rpcd -t 30
Device.DeviceInfo.ProcessStatus.Process.50.PID => 2898
Device.DeviceInfo.ProcessStatus.Process.50.Priority => 20
Device.DeviceInfo.ProcessStatus.Process.50.Size => 1504
Device.DeviceInfo.ProcessStatus.Process.50.State => Sleeping
Device.DeviceInfo.ProcessStatus.Process.52.CPUTime => 0
Device.DeviceInfo.ProcessStatus.Process.52.Command => /sbin/questd
Device.DeviceInfo.ProcessStatus.Process.52.PID => 3003
Device.DeviceInfo.ProcessStatus.Process.52.Priority => 20
Device.DeviceInfo.ProcessStatus.Process.52.Size => 1692
Device.DeviceInfo.ProcessStatus.Process.52.State => Running
Device.DeviceInfo.ProcessStatus.Process.53.CPUTime => 1000
Device.DeviceInfo.ProcessStatus.Process.53.Command => swmdk
Device.DeviceInfo.ProcessStatus.Process.53.PID => 3225
Device.DeviceInfo.ProcessStatus.Process.53.Priority => 20
Device.DeviceInfo.ProcessStatus.Process.53.Size => 18528
Device.DeviceInfo.ProcessStatus.Process.53.State => Sleeping
Device.DeviceInfo.ProcessStatus.Process.58.CPUTime => 0
Device.DeviceInfo.ProcessStatus.Process.58.Command => /sbin/dslmngr
Device.DeviceInfo.ProcessStatus.Process.58.PID => 3880
Device.DeviceInfo.ProcessStatus.Process.58.Priority => 20
Device.DeviceInfo.ProcessStatus.Process.58.Size => 1624
Device.DeviceInfo.ProcessStatus.Process.58.State => Sleeping
Device.DeviceInfo.ProcessStatus.Process.59.CPUTime => 0
Device.DeviceInfo.ProcessStatus.Process.59.Command => /sbin/netifd
Device.DeviceInfo.ProcessStatus.Process.59.PID => 4156
Device.DeviceInfo.ProcessStatus.Process.59.Priority => 20
Device.DeviceInfo.ProcessStatus.Process.59.Size => 1528
Device.DeviceInfo.ProcessStatus.Process.59.State => Sleeping
Device.DeviceInfo.ProcessStatus.Process.62.CPUTime => 33000
Device.DeviceInfo.ProcessStatus.Process.62.Command => endptmngr
Device.DeviceInfo.ProcessStatus.Process.62.PID => 4397
Device.DeviceInfo.ProcessStatus.Process.62.Priority => 8
Device.DeviceInfo.ProcessStatus.Process.62.Size => 6912
Device.DeviceInfo.ProcessStatus.Process.62.State => Sleeping
Device.DeviceInfo.ProcessStatus.Process.65.CPUTime => 0
Device.DeviceInfo.ProcessStatus.Process.65.Command => /sbin/ntpd -g -u ntp:ntp -p /var/run/ntpd.pid -n -c /var/etc/ntpd.conf
Device.DeviceInfo.ProcessStatus.Process.65.PID => 5281
Device.DeviceInfo.ProcessStatus.Process.65.Priority => 20
Device.DeviceInfo.ProcessStatus.Process.65.Size => 3336
Device.DeviceInfo.ProcessStatus.Process.65.State => Sleeping
Device.DeviceInfo.ProcessStatus.Process.71.CPUTime => 14000
Device.DeviceInfo.ProcessStatus.Process.71.Command => /usr/sbin/uspd
Device.DeviceInfo.ProcessStatus.Process.71.PID => 5896
Device.DeviceInfo.ProcessStatus.Process.71.Priority => 20
Device.DeviceInfo.ProcessStatus.Process.71.Size => 8100
Device.DeviceInfo.ProcessStatus.Process.71.State => Sleeping
Device.DeviceInfo.ProcessStatus.Process.72.CPUTime => 0
Device.DeviceInfo.ProcessStatus.Process.72.Command => /usr/bin/owsd -f /tmp/owsd/owsd.json
Device.DeviceInfo.ProcessStatus.Process.72.PID => 6013
Device.DeviceInfo.ProcessStatus.Process.72.Priority => 20
Device.DeviceInfo.ProcessStatus.Process.72.Size => 3884
Device.DeviceInfo.ProcessStatus.Process.72.State => Sleeping
Device.DeviceInfo.ProcessStatus.Process.73.CPUTime => 0
Device.DeviceInfo.ProcessStatus.Process.73.Command => /usr/sbin/ethmngr
Device.DeviceInfo.ProcessStatus.Process.73.PID => 6037
Device.DeviceInfo.ProcessStatus.Process.73.Priority => 20
Device.DeviceInfo.ProcessStatus.Process.73.Size => 4392
Device.DeviceInfo.ProcessStatus.Process.73.State => Sleeping
Device.DeviceInfo.ProcessStatus.Process.74.CPUTime => 0
Device.DeviceInfo.ProcessStatus.Process.74.Command => /usr/sbin/mcpd
Device.DeviceInfo.ProcessStatus.Process.74.PID => 6302
Device.DeviceInfo.ProcessStatus.Process.74.Priority => -6
Device.DeviceInfo.ProcessStatus.Process.74.Size => 3688
Device.DeviceInfo.ProcessStatus.Process.74.State => Sleeping
Device.DeviceInfo.ProcessStatus.Process.75.CPUTime => 0
Device.DeviceInfo.ProcessStatus.Process.75.Command => /usr/sbin/asterisk -C /var/etc/asterisk/asterisk.conf
Device.DeviceInfo.ProcessStatus.Process.75.PID => 6809
Device.DeviceInfo.ProcessStatus.Process.75.Priority => 20
Device.DeviceInfo.ProcessStatus.Process.75.Size => 21824
Device.DeviceInfo.ProcessStatus.Process.75.State => Sleeping
Device.DeviceInfo.ProcessStatus.Process.76.CPUTime => 0
Device.DeviceInfo.ProcessStatus.Process.76.Command => astcanary /var/run/asterisk/alt.asterisk.canary.tweet.tweet.tweet 6809
Device.DeviceInfo.ProcessStatus.Process.76.PID => 6810
Device.DeviceInfo.ProcessStatus.Process.76.Priority => 20
Device.DeviceInfo.ProcessStatus.Process.76.Size => 1848
Device.DeviceInfo.ProcessStatus.Process.76.State => Sleeping
Device.DeviceInfo.ProcessStatus.Process.77.CPUTime => 0
Device.DeviceInfo.ProcessStatus.Process.77.Command => /usr/sbin/obuspa -v 1 -l /var/log/obuspa -r /tmp/obuspa_param_reset.txt
Device.DeviceInfo.ProcessStatus.Process.77.PID => 6911
Device.DeviceInfo.ProcessStatus.Process.77.Priority => 20
Device.DeviceInfo.ProcessStatus.Process.77.Size => 7496
````

#### validate_2912_1

##### Description

Tests the obuspa method `add` for test case 2912_1, 
for path "Device.DHCPv4.Server.Pool."
[add](./functionspec.md#add).

##### Test Steps

Issuing a add to a client from the obuspa test platform, with cli option.

Read the logfile and verify that the output argument was
accurately logged.

##### Test Expected Results

The expected result is for the log file to have recorded a call to the
add handler function in obuspa.

````bash
Added Device.DHCPv4.Server.Pool.1
````

#### validate_2912_3

##### Description

Tests the obuspa method `set` for test case 2912_3, 
for path "Device.DHCPv4.Server.Pool.1.StaticAddress.1.Yiaddr" and value "192.268.0.123"
[set](./functionspec.md#set).

##### Test Steps

Issuing a set to a client from the obuspa test platform, with cli option.

Read the logfile and verify that the output argument was
accurately logged.

##### Test Expected Results

The expected result is for the log file to have recorded a call to the
set handler function in obuspa.

````bash
Device.DHCPv4.Server.Pool.1.StaticAddress.1.Yiaddr => 192.268.0.123
````
